<?php
//
// Microsoft Exchange Config Wizard
// Copyright (c) 2011-2020 Nagios Enterprises, LLC. All rights reserved.
//

include_once(dirname(__FILE__).'/../configwizardhelper.inc.php');

exchange_configwizard_init();

function exchange_configwizard_init()
{
    $name = "exchange";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "2.0.0",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor a Microsoft&reg; Exchange server."),
        CONFIGWIZARD_DISPLAYTITLE => "Exchange Server",
        CONFIGWIZARD_FUNCTION => "exchange_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "exchange2010.png",
        CONFIGWIZARD_FILTER_GROUPS => array('windows','email'),
        CONFIGWIZARD_REQUIRES_VERSION => 500
    );
    register_configwizard($name, $args);
}

/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function exchange_configwizard_func($mode = "", $inargs = null, &$outargs, &$result)
{
    $wizard_name = "exchange";

    #$agent_url = "http://www.nsclient.org/download/";
    #$agent32_stable_url = "https://assets.nagios.com/downloads/nagiosxi/agents/NSClient++/NSCP-0.4.4.23-Win32.msi";
    #$agent64_stable_url = "https://assets.nagios.com/downloads/nagiosxi/agents/NSClient++/NSCP-0.4.4.23-x64.msi";

    /* Prerequisite software */
    $NCPA_download_url = "https://www.nagios.org/ncpa/#downloads";
    $NCPA_docs_url = "https://www.nagios.org/ncpa/#docs";

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:

            $ip_address = grab_array_var($inargs, "ip_address", "");
            $version = grab_array_var($inargs, "version", "2019");
            $port = grab_array_var($inargs, "port", "5693");
            $token = grab_array_var($inargs, "token", "");

            $output = '
<h5 class="ul">'._('Exchange Server - NCPA Setup').'</h5>
<p><i>'._('The agent should be installed before running this wizard.').'</i></p>
<ul style="padding: 0 0 0 30px;">
    <li><a href="'.$NCPA_download_url.'" target="_blank">'._('Download the latest version of NCPA').'</a> '._('for the system you would like to monitor').'</li>
    <li>'._('Follow the').' <a href="'.$NCPA_docs_url.'" target="_blank">'._('installation instructions').'</a>'._(' and configure the token for the agent').'</li>
</ul>

<h5 class="ul">'._("Exchange Server Information").'</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td class="vt">
            <label>'._('IP Address').':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="'.encode_form_val($ip_address).'" class="textfield form-control">
            <div class="subtext">'._('The IP address of the Exchange server you\'d like to monitor').'.</div>
        </td>
    </tr>
    <tr>
        <td>
            <label>'._('Version').':</label>
        </td>
        <td>
            <select name="version" class="form-control">
                <option value="5.5" '.is_selected($version, "5.5").'>Exchange 5.5</option>
                <option value="2000" '.is_selected($version, "2000").'>Exchange 2000</option>
                <option value="2003" '.is_selected($version, "2003").'>Exchange 2003</option>
                <option value="2007" '.is_selected($version, "2007").'>Exchange 2007</option>
                <option value="2010" '.is_selected($version, "2010").'>Exchange 2010</option>
                <option value="2013" '.is_selected($version, "2013").'>Exchange 2013</option>
                <option value="2016" '.is_selected($version, "2016").'>Exchange 2016</option>
                <option value="2019" '.is_selected($version, "2019").'>Exchange 2019</option>
            </select>
        </td>
    </tr>
    <tr>
        <td class="vt"><label>'._('NCPA Port').':</label></td>
        <td>
            <input type="text" size="5" name="port" value="'.encode_form_val($port).'" class="textfield usermacro-detection form-control">
            <div class="subtext">'._('Port used to connect to NCPA. Defaults to port 5693').'.</div>
        </td>
    </tr>
    <tr>
        <td class="vt"><label>'._('NCPA Token').':</label></td>
        <td>
            <input type="text" size="20" name="token" id="token" value="'.encode_form_val($token).'" class="textfield usermacro-detection form-control" autocomplete="off">
            <div class="subtext">'._('Authentication token used to connect to NCPA').'.</div>
        </td>
    </tr>
</table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
            $ip_address = grab_array_var($inargs, "ip_address", "");
            $ip_address_replaced = nagiosccm_replace_user_macros($ip_address);

            $version = grab_array_var($inargs, "version", "");

            // NCPA
            $port = grab_array_var($inargs, "port", "5693");
            $token = grab_array_var($inargs, "token", "");

            $port_replaced = nagiosccm_replace_user_macros($port);
            $token_replaced = nagiosccm_replace_user_macros($token);

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (have_value($ip_address_replaced) == false) {
                $errmsg[$errors++] = _('No address specified.');
            } else if (!valid_ip($ip_address_replaced)) {
                $errmsg[$errors++] = _('Invalid IP address.');
            }

            if (have_value($version) == false) {
                $errmsg[$errors++] = _('No server version specified.');
            }

            if (have_value($token) == false) {
                $errmsg[$errors++] = _('No NCPA Token specified.');
            }

            // Test the connection if no errors
            if (empty($errors)) {
                // The URL we will use to query the NCPA agent, and do a walk
                // of all monitorable items.
                $query_url = "https://{$ip_address}:{$port}/testconnect/?token=".urlencode($token);
                $query_url_replaced = "https://{$ip_address_replaced}:{$port_replaced}/testconnect/?token=".urlencode($token_replaced);

                $context = array("ssl" => array("verify_peer" => false, "verify_peer_name" => false));

                // All we want to do is test if we can hit this URL.
                $raw_json = file_get_contents($query_url_replaced, false, stream_context_create($context));

                if (empty($raw_json)) {
                    $errmsg[$errors++] = _("Unable to contact server at")." {$query_url}.";

                } else {
                    $json = json_decode($raw_json, true);

                    if (!array_key_exists('value', $json)) {
                        $errmsg[$errors++] = _("Bad token for connection.");
                    }
                }
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us
            $ip_address = grab_array_var($inargs, "ip_address");
            $version = grab_array_var($inargs, "version");

            // NCPA
            $port = grab_array_var($inargs, "port");
            $token = grab_array_var($inargs, "token");

            $hn = @gethostbyaddr($ip_address);

            if ($hn == "")  $hn = $ip_address;

            $hostname = grab_array_var($inargs, "hostname", $hn);
            $password = grab_array_var($inargs, "password", "");
            $url = grab_array_var($inargs, "url", "http://".$hostname."/exchange/");

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial", "");

            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));
            }

            if (!is_array($services)) {
                $services_default = array(
                    "ping" => 1,
                    "smtp" => 1,
                    "imap" => 1,
                    "pop" => 1,
                    "rbl" => 1,
                    "owa_http" => 1,
                    "owa_https" => 0,
                    "core_services" => 1,
                    "web_services" => 1,
                    "pending_routing" => 0,     # Exchange 2003 - unchecked
                    "remote_queue_length" => 0, # Exchange 2003 - unchecked
                    "messages_queued_for_delivery" => 0,    # Exchange 2007 - 2019 - unchecked
                );

                $services = grab_array_var($inargs, "services", $services_default);
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            if ($serviceargs_serial != "") {
                $serviceargs = unserialize(base64_decode($serviceargs_serial));
            }

            if (!is_array($serviceargs)) {
                $serviceargs_default = array(
                    "rbl_servers" => "zen.spamhaus.org bl.spamcop.net dnsbl.ahbl.org dnsbl.njabl.org dnsbl.sorbs.net virbl.dnsbl.bit.nl rbl.efnet.org phishing.rbl.msrbl.net 0spam.fusionzero.com list.dsbl.org multihop.dsbl.org unconfirmed.dsbl.org will-spam-for-food.eu.org blacklist.spambag.org blackholes.brainerd.net blackholes.uceb.org spamsources.dnsbl.info map.spam-rbl.com ns1.unsubscore.com psbl.surriel.com l2.spews.dnsbl.sorbs.net bl.csma.biz sbl.csma.biz dynablock.njabl.org no-more-funn.moensted.dk  ubl.unsubscore.com dnsbl-1.uceprotect.net dnsbl-2.uceprotect.net dnsbl-3.uceprotect.net spamguard.leadmon.net opm.blitzed.org bl.spamcannibal.org rbl.schulte.org dnsbl.ahbl.org virbl.dnsbl.bit.nl combined.rbl.msrbl.net",
                    "core_service_names" => "",
                    "web_service_names" => "W3SVC",
                    "pending_routing_warning" => 25,
                    "pending_routing_critical" => 100,
                    "remote_queue_length_warning" => 25,
                    "remote_queue_length_critical" => 50,
                    "messages_queued_for_delivery_warning" => 25,
                    "messages_queued_for_delivery_critical" => 50,
                );

                switch ($version) {
                    case "2019":
                        // assumed for 2019 - Exchange Services List (one additional to 2016 list)
                        // Verified against: https://docs.microsoft.com/en-us/exchange/plan-and-deploy/deployment-ref/services-overview?view=exchserver-2019
                        $serviceargs_default["core_service_names"] = "HostControllerService,MSComplianceAudit,MSExchangeADTopology,MSExchangeCompliance,MSExchangeDagMgmt,MSExchangeDelivery,MSExchangeDiagnostics,MSExchangeFastSearch,MSExchangeFrontEndTransport,MSExchangeHM,MSExchangeHMRecovery,MSExchangeIS,MSExchangeMailboxAssistants,MSExchangeMailboxReplication,MSExchangeNotificationsBroker,MSExchangeRepl,MSExchangeRPC,MSExchangeServiceHost,MSExchangeSubmission,MSExchangeThrottling,MSExchangeTransport,SearchExchangeTracing,";
                        // Optional, but setup as automatic during default install.
                        $serviceargs_default["core_service_names"] .= "MSExchangeAntispamUpdate,MSExchangeEdgeSync,MSExchangeTransportLogSearch,MSExchangeUM,MSExchangeUMCR";
                            // These are manual services in 2019 and cause a CRITICAL return if included: MSExchangeImap4,MSExchangeIMAP4BE,MSExchangePop3,MSExchangePOP3BE,WSBExchange
                        break;
                    case "2016":
                        // assumed for 2016 - Exchange Services List 
                        // Verified against: https://docs.microsoft.com/en-us/exchange/plan-and-deploy/deployment-ref/services-overview?view=exchserver-2016
                        // sc query type= service state= all > allserviceslist.txt ==> search for "Microsoft Exchange" to find the services.
                        $serviceargs_default["core_service_names"] = "HostControllerService,MSComplianceAudit,MSExchangeADTopology,MSExchangeCompliance,MSExchangeDagMgmt,MSExchangeDelivery,MSExchangeDiagnostics,MSExchangeFastSearch,MSExchangeFrontEndTransport,MSExchangeHM,MSExchangeHMRecovery,MSExchangeIS,MSExchangeMailboxAssistants,MSExchangeMailboxReplication,MSExchangeRepl,MSExchangeRPC,MSExchangeServiceHost,MSExchangeSubmission,MSExchangeThrottling,MSExchangeTransport,SearchExchangeTracing,";
                        // Optional, but setup as automatic during default install.
                        $serviceargs_default["core_service_names"] .= "MSExchangeAntispamUpdate,MSExchangeEdgeSync,MSExchangeTransportLogSearch,MSExchangeUM,MSExchangeUMCR";
                            // These are manual services in 2016 and cause a CRITICAL return if included: MSExchangeImap4,MSExchangeIMAP4BE,MSExchangePop3,MSExchangePOP3BE,MSExchangeNotificationsBroker,WSBExchange
                        break;
                    case "2013":
                        // assumed for 2013 - nagioswiki.com
                        $serviceargs_default["core_service_names"] = "MSExchangeADTopology,MSExchangeAntispamUpdate,MSExchangeDagMgmt,MSExchangeDelivery,MSExchangeDiagnostics,MSExchangeEdgeSync,MSExchangeFastSearch,MSExchangeFrontEndTransport,MSExchangeHM,MSExchangeImap4,MSExchangeIMAP4BE,MSExchangeIS,MSExchangeMailboxAssistants,MSExchangeMailboxReplication,MSExchangePop3,MSExchangePOP3BE,MSExchangeRepl,MSExchangeRPC,MSExchangeServiceHost,MSExchangeSubmission,MSExchangeThrottling,MSExchangeTransport,MSExchangeTransportLogSearch,MSExchangeUM,MSExchangeUMCR";
                        break;
                    case "2010":
                        // assumed for 2010 - nagioswiki.com
                        $serviceargs_default["core_service_names"] = "MSExchangeADTopology,MSExchangeAntispamUpdate,MSExchangeEdgeSync,MSExchangeFDS,MSExchangeImap4,MSExchangeIS,MSExchangeMailboxAssistants,MSExchangeMailSubmission,MSExchangeMonangePop3,MSExchangeRepl,MSExchangeSA,MSExchangeSearch,MSExchangeServiceHost,MSExchangeTransport,MSExchangeTransportLogSearch,msftesql-Exchange";
                        break;
                    case "2007":
                        // known for 2007 - nagioswiki.com
                        $serviceargs_default["core_service_names"] = "MSExchangeADTopology,MSExchangeAntispamUpdate,MSExchangeEdgeSync,MSExchangeFDS,MSExchangeImap4,MSExchangeIS,MSExchangeMailboxAssistants,MSExchangeMailSubmission,MSExchangeMonangePop3,MSExchangeRepl,MSExchangeSA,MSExchangeSearch,MSExchangeServiceHost,MSExchangeTransport,MSExchangeTransportLogSearch,msftesql-Exchange";
                        break;
                    case "2003":
                        // assumed for 2003 - nagioswiki.com
                        $serviceargs_default["core_service_names"] = "MSExchangeIS,MSExchangeMTA,SMTPSVC,RESvc";
                    case "2000":
                        // known for 2000 - nagioswiki.com
                        $serviceargs_default["core_service_names"] = "MSExchangeIS,MSExchangeMTA,SMTPSVC,RESvc";
                        break;
                    case "5.5":
                    default:
                        // known for 5.5 - Nagios Exchange (http://exchange.nagios.org/directory/Plugins/Email-and-Groupware/Microsoft-Exchange/Check-MS-Exchange-Server-Health/details)
                        $serviceargs_default["core_service_names"] = "MSExchangeDS,MSExchangeES,MSExchangeIMC,MSExchangeIS,MSExchangeMTA,MSExchangeSA ";
                        break;
                }

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            $output = '     
<input type="hidden" name="ip_address" value="'.encode_form_val($ip_address).'">
<input type="hidden" name="version" value="'.encode_form_val($version).'">
<input type="hidden" name="port" value="'.encode_form_val($port).'">
<input type="hidden" name="token" value="'.encode_form_val($token).'">

<h5 class="ul">'._('Exchange Server Details').'</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td>
            <label>'._('IP Address').':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="'.encode_form_val($ip_address).'" class="textfield form-control" disabled>
        </td>
    </tr>
    <tr>
        <td>
            <label>'._('Server Version').':</label>
        </td>
            <td><b>Exchange '.encode_form_val($version).'</b><br>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>'._('Host Name').':</label>
        </td>
        <td>
            <input type="text" size="20" name="hostname" id="hostname" value="'.encode_form_val($hostname).'" class="textfield form-control">
            <div class="subtext">'._('The name you\'d like to have associated with this Exchange server').'.</div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>'._('URL').':</label>
        </td>
        <td>
            <input type="text" size="40" name="url" id="url" value="'.encode_form_val($url).'" class="textfield form-control">
            <div class="subtext">'._('The URL used to access OWA on the Exchange server').'.</div>
        </td>
    </tr>
</table>

<h5 class="ul">'._('Basic Services').'</h5>
<p>'._('Specify which services you\'d like to monitor for the Exchange server').'.</p>
<table class="table table-no-border table-auto-width">
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[ping]" '.is_checked(checkbox_binary($services["ping"]), "1").'>
        </td>
        <td>
            <b>'._('Ping').'</b><br>
            '._('Monitors the server with an ICMP ping.  Useful for watching network latency and general uptime').'.
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[smtp]" '.is_checked(checkbox_binary($services["smtp"]), "1").'>
        </td>
        <td>
            <b>'._('SMTP').'</b><br>
            '._('Monitors SMTP service availability').'.
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[imap]" '.is_checked(checkbox_binary($services["imap"]), "1").'>
        </td>
        <td>
            <b>'._('IMAP').'</b><br>
            '._('Monitors IMAP service availability').'.
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[pop]" '.is_checked(checkbox_binary($services["pop"]), "1").'>
        </td>
        <td>
            <b>'._('POP').'</b><br>
            '._('Monitors POP service availability').'.
        </td>
    </tr>
    <tr>
        <td class="vt">
            <input type="checkbox" class="checkbox" name="services[rbl]" '.is_checked(checkbox_binary($services["rbl"]), "1").'>
        </td>
        <td>
            <b>'._('RBL Blacklist Check').'</b><br>
            '._('Checks to see if your mail server is listed on any public RBLs (real time blackhole lists)').'.<br>
            <div class="pad-t5">
                <label>'._('Blacklist Servers').':</label>
                <input type="text" size="60" name="serviceargs[rbl_servers]" value="'.encode_form_val($serviceargs["rbl_servers"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[owa_http]" '.is_checked(checkbox_binary($services["owa_http"]), "1").'>
        </td>
        <td>
            <b>'._('OWA HTTP').'</b><br>
            '._('Monitors the availability of Outlook Web Access over HTTP').'.
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[owa_https]" '.is_checked(checkbox_binary($services["owa_https"]), "1").'>
        </td>
        <td>
            <b>'._('OWA HTTPS').'</b><br>
            '._('Monitors the availability of Outlook Web Access over HTTPS (secured with SSL)').'.
        </td>
    </tr>
</table>

<h5 class="ul">'._('Exchange Services').'</h5>
<p>'._('Specify which Exchange services you\'d like to monitor (these require installation of the Windows agent)').'.<br><b>'._('You must have the listed core service classes available for the wizard services to run correctly.').'</b></p>
<table class="table table-no-border table-auto-width">
    <tr>
        <td class="vt">
            <input type="checkbox" class="checkbox" name="services[core_services]" '.is_checked(checkbox_binary($services["core_services"]), "1").'>
        </td>
        <td>
            <b>'._('Core Services').'</b><br>
            '._('Checks to make sure core services (specified below) that are essential to Exchange are running').'.
            <div class="pad-t5">
                <label>'._('Services').':</label>
                <input type="text" size="40" name="serviceargs[core_service_names]" value="'.encode_form_val($serviceargs["core_service_names"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <input type="checkbox" class="checkbox" name="services[web_services]" '.is_checked(checkbox_binary($services["web_services"]), "1").'>
        </td>
        <td>
            <b>'._('Web Services').'</b><br>
            '._('Checks to make sure web services (specified below) that are essential to Exchange are running').'.
            <div class="pad-t5">
                <label>'._('Services').':</label>
                <input type="text" size="10" name="serviceargs[web_service_names]" value="'.encode_form_val($serviceargs["web_service_names"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
</table>

<h5 class="ul">'._('Exchange Metrics').'</h5>
<p>'._('Specify which metrics you\'d like to monitor on the Exchange server (these require installation of the Windows agent)').'.</p>
<table class="table table-no-border table-auto-width">
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[pending_routing]"  '.is_checked(checkbox_binary($services["pending_routing"]), "1").'>
        </td>
        <td>
            <b>'._('Messages Pending Routing').'</b>
            <div class="pad-t5">
                <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>  <input type="text" size="3" name="serviceargs[pending_routing_warning]" value="'.encode_form_val($serviceargs["pending_routing_warning"]).'" class="textfield form-control condensed"> &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="3" name="serviceargs[pending_routing_critical]" value="'.encode_form_val($serviceargs["pending_routing_critical"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[remote_queue_length]"  '.is_checked(checkbox_binary($services["remote_queue_length"]), "1").'>
        </td>
        <td>
            <b>'._('Remote Queue Length').'</b>
            <div class="pad-t5">
                <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>  <input type="text" size="3" name="serviceargs[remote_queue_length_warning]" value="'.encode_form_val($serviceargs["remote_queue_length_warning"]).'" class="textfield form-control condensed"> &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="3" name="serviceargs[remote_queue_length_critical]" value="'.encode_form_val($serviceargs["remote_queue_length_critical"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" class="checkbox" name="services[messages_queued_for_delivery]"  '.is_checked(checkbox_binary($services["messages_queued_for_delivery"]), "1").'>
        </td>
        <td>
            <b>'._('Messages Queued For Delivery Total').'</b>
            <div class="pad-t5">
                <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label>  <input type="text" size="3" name="serviceargs[messages_queued_for_delivery_warning]" value="'.encode_form_val($serviceargs["messages_queued_for_delivery_warning"]).'" class="textfield form-control condensed"> &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="3" name="serviceargs[messages_queued_for_delivery_critical]" value="'.encode_form_val($serviceargs["messages_queued_for_delivery_critical"]).'" class="textfield form-control condensed">
            </div>
        </td>
    </tr>
</table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // Get variables that were passed to us
            $ip_address = grab_array_var($inargs, "ip_address");
            $version = grab_array_var($inargs, "version", "");
            $hostname = grab_array_var($inargs, "hostname");
            $hostname_replaced = nagiosccm_replace_user_macros($hostname);

            $password = grab_array_var($inargs, "password");
            $password_replaced = nagiosccm_replace_user_macros($password);

            $url = grab_array_var($inargs, "url", "");
            $url_replaced = nagiosccm_replace_user_macros($url);

            // NCPA
            $port = grab_array_var($inargs, "port");
            $token = grab_array_var($inargs, "token");

            $port_replaced = nagiosccm_replace_user_macros($port);
            $token_replaced = nagiosccm_replace_user_macros($token);

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (is_valid_host_name($hostname_replaced) == false) {
                $errmsg[$errors++] = "Invalid host name.";
            }

            if (preg_match('/[^a-zA-Z0-9 .\:_-@]/', $password_replaced)) {
                $errmsg[$errors++] = _("Password contains invalid characters.");
            }

            if (valid_url($url_replaced) == false) {
                $errmsg[$errors++] = _("Invalid URL.");
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // Get variables that were passed to us
            $ip_address = grab_array_var($inargs, "ip_address");
            $version = grab_array_var($inargs, "version", "");
            $hostname = grab_array_var($inargs, "hostname");
            $password = grab_array_var($inargs, "password");
            $url = grab_array_var($inargs, "url", "");

            // NCPA
            $port = grab_array_var($inargs, "port");
            $token = grab_array_var($inargs, "token");

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");

            if ($services_serial != "") {
                $services = unserialize(base64_decode($services_serial));

            } else {
                $services = grab_array_var($inargs, "services");
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");

            if ($serviceargs_serial != "") {
                $serviceargs = unserialize(base64_decode($serviceargs_serial));

            } else {
                $serviceargs = grab_array_var($inargs, "serviceargs");
            }

            $output = '
            
        <input type="hidden" name="ip_address" value="'.encode_form_val($ip_address).'">
        <input type="hidden" name="version" value="'.encode_form_val($version).'">
        <input type="hidden" name="port" value="'.encode_form_val($port).'">
        <input type="hidden" name="token" value="'.encode_form_val($token).'">
        <input type="hidden" name="url" value="'.encode_form_val($url).'">
        <input type="hidden" name="hostname" value="'.encode_form_val($hostname).'">
        <input type="hidden" name="password" value="'.encode_form_val($password).'">
        <input type="hidden" name="services_serial" value="'.base64_encode(serialize($services)).'">
        <input type="hidden" name="serviceargs_serial" value="'.base64_encode(serialize($serviceargs)).'">
        
        <!--SERVICES='.serialize($services).'<BR>
        SERVICEARGS='.serialize($serviceargs).'<BR>-->
        
            ';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            $hostname = grab_array_var($inargs, "hostname", "");
            $version = grab_array_var($inargs, "version", "");
            $ip_address = grab_array_var($inargs, "ip_address", "");
            $password = grab_array_var($inargs, "password", "");
            $hostaddress = $ip_address;
            $url = grab_array_var($inargs, "url", "");

            // NCPA
            $port = grab_array_var($inargs, "port", "");
            $token = grab_array_var($inargs, "token", "");

            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = unserialize(base64_decode($services_serial));
            $serviceargs = unserialize(base64_decode($serviceargs_serial));

            // Escape values for check_command line
            if (function_exists('nagiosccm_replace_command_line')) {
                $token = nagiosccm_replace_command_line($token);
            } else {
                $token = str_replace('!', '\!', $token);
            }

            /*
            echo "SERVICES<BR>";
            print_r($services);
            echo "<BR>";
            echo "SERVICEARGS<BR>";
            print_r($serviceargs);
            echo "<BR>";
            */

            // save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["ip_address"] = $ip_address;
            $meta_arr["password"] = $password;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_exchange_host",
                    "host_name" => $hostname,
                    "ip_address" => $hostaddress,
                    "icon_image" => "exchange2010.png",
                    "statusmap_image" => "exchange2010.png",
                    "_xiwizard" => $wizard_name,
                );
            }

            $commonopts = "-t '$token' -P '$port'";

            // see which services we should monitor
            foreach ($services as $svc => $svcstate) {

                //echo "PROCESSING: $svc -> $svcstate<BR>\n";

                switch ($svc) {

                    case "ping":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Ping",
                            "use" => "xiwizard_exchange_ping_service",
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "imap":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "IMAP",
                            "use" => "xiwizard_imap_service",
                            "check_command" => "check_xi_service_imap!-j",
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "pop":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "POP",
                            "use" => "xiwizard_pop_service",
                            "check_command" => "check_xi_service_pop!-j",
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "smtp":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "SMTP",
                            "use" => "xiwizard_smtp_service",
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "rbl":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Blacklist Status",
                            "use" => "xiwizard_exchange_service",
                            "check_command" => "check_exchange_rbl!-B ".$serviceargs["rbl_servers"],
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "owa_http":
                    case "owa_https":

                        $pluginopts = "";

                        $urlparts = parse_url($url);
                        $vhost = $urlparts["host"];

                        if ($vhost == "") {
                            $vhost = $ip_address;
                        }

                        $pluginopts .= " -H ".$vhost; // virtual host name

                        $pluginopts .= " -f ok"; // on redirect, follow (OK status)
                        $pluginopts .= " -I ".$ip_address; // ip address

                        $urlpath = $urlparts["path"];

                        if ($urlpath == "") {
                            $urlpath = "/";
                        }

                        $pluginopts .= " -u \"".$urlpath."\"";

                        if ($svc == "owa_https") {
                            $pluginopts .= " -S";
                        }

                        if (!empty($port)) {
                            $pluginopts .= " -p ".$port;
                        }

                        if (!empty($username)) {
                            $pluginopts .= " -a \"".$username.":".$password."\"";
                        }

                        $sname = "OWA HTTP";

                        if ($svc == "owa_https") {
                            $sname .= "S";
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => $sname,
                            "use" => "xiwizard_exchange_service",
                            "check_command" => "check_xi_service_http!".$pluginopts,
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    case "core_services":

                        $coreServices = explode(",", $serviceargs["core_service_names"]);
                        $coreServiceArgs = "";

                        foreach ($coreServices as $i => $service) {
                            $coreServiceArgs .= "service=$service".(($i + 1 < count($coreServices)) ? "," : "");
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Exchange Core Services",
                            #"use" => "xiwizard_exchange_service",
                            "use" => "xiwizard_ncpa_service",
                            #"check_command" => "check_xi_service_nsclient!".$password."!SERVICESTATE!-l ".$serviceargs["core_service_names"]." -d SHOWALL",
                            "check_command" => "check_xi_ncpa!".$commonopts." -M 'services' -q '".$coreServiceArgs.",status=running'",
                            "_xiwizard" => $wizard_name);

                        break;

                    case "web_services":

                        $webServices = explode(",", $serviceargs["web_service_names"]);
                        $webServiceArgs = "";

                        foreach ($webServices as $i => $service) {
                            $webServiceArgs .= "service=$service".(($i + 1 < count($webServices)) ? "," : "");
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Exchange Web Services for: ".$service,
                            #"use" => "xiwizard_exchange_service",
                            "use" => "xiwizard_ncpa_service",
                            #"check_command" => "check_xi_service_nsclient!".$password."!SERVICESTATE!-l ".$serviceargs["web_service_names"]." -d SHOWALL",
                            "check_command" => "check_xi_ncpa!".$commonopts." -M 'services' -q '".$webServiceArgs.",status=running'",
                            "_xiwizard" => $wizard_name);

                        break;

                    # 2003 Exchange
                    case "pending_routing":

                        #$checkcommand = "check_xi_service_nsclient!".$password."!COUNTER!-l \"\\\\SMTP Server(_Total)\\\\Messages Pending Routing\"";
                        $checkcommand = "check_xi_ncpa!".$commonopts." -M 'windowscounters/SMTP Server(_Total)/Messages Pending Routing'";

                        if ($serviceargs["pending_routing_warning"] != "") {
                            $checkcommand .= " -w ".$serviceargs["pending_routing_warning"];
                        }

                        if ($serviceargs["pending_routing_critical"] != "") {
                            $checkcommand .= " -c ".$serviceargs["pending_routing_critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Messages Pending Routing",
                            #"use" => "xiwizard_windowsserver_nsclient_service",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => $checkcommand,
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    # 2003 Exchange
                    case "remote_queue_length":

                        #$checkcommand = "check_xi_service_nsclient!".$password."!COUNTER!-l \"\\\\SMTP Server(_Total)\\\\Remote Queue Length\"";
                        $checkcommand = "check_xi_ncpa!".$commonopts." -M 'windowscounters/SMTP Server(_Total)/Remote Queue Length'";

                        if ($serviceargs["remote_queue_length_warning"] != "") {
                            $checkcommand .= " -w ".$serviceargs["remote_queue_length_warning"];
                        }

                        if ($serviceargs["remote_queue_length_critical"] != "") {
                            $checkcommand .= " -c ".$serviceargs["remote_queue_length_critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Remote Queue Length",
                            #"use" => "xiwizard_windowsserver_nsclient_service",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => $checkcommand,
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    # 2007 - 2019 Exchange
                    case "messages_queued_for_delivery":

                        $checkcommand = "check_xi_ncpa!".$commonopts." -M 'windowscounters/MSExchangeTransport Queues(_Total)/Messages Queued for Delivery Total'";

                        if ($serviceargs["messages_queued_for_delivery_warning"] != "") {
                            $checkcommand .= " -w ".$serviceargs["messages_queued_for_delivery_warning"];
                        }

                        if ($serviceargs["messages_queued_for_delivery_critical"] != "") {
                            $checkcommand .= " -c ".$serviceargs["messages_queued_for_delivery_critical"];
                        }

                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Messages Queued for Delivery Total",
                            #"use" => "xiwizard_windowsserver_nsclient_service",
                            "use" => "xiwizard_ncpa_service",
                            "check_command" => $checkcommand,
                            "_xiwizard" => $wizard_name,
                        );

                        break;

                    default:
                        break;
                }
            }

            //echo "OBJECTS:<BR>";
            //print_r($objs);
            //exit();

            // return the object definitions to the wizard
            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}

?>

