<?php

error_reporting(E_ALL);
ini_set('display_errors', 1);


//
// Audit Log (Enterprise Feature)
// Copyright (c) 2008-2018 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/../../common.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables and check prereqs
grab_request_vars();
check_prereqs();
check_authentication(false);

route_request();

function route_request()
{
    $mode = grab_request_var("mode", "");
    switch ($mode) {
        case "audio-upload":
            do_upload_audio();
            break;
        case "load-audio":
            do_load_audio();
            break;
        case "delete-audio":
            do_delete_audio();
            break;
        case 'save-audio-prefs':
            do_save_audio_prefs();
            break;
        case 'get-audio-prefs':
            do_get_audio_prefs();
        default:
            break;
    }
}

function do_upload_audio() {

    foreach($_FILES['audio']['error'] as $error) {
        if ($error != 0) {
            print json_encode(array(
                'error' => true,
                'errmsg' => _('Could not transfer file to server.')
            ));
            return;
        }
    }

    $tmp_loc = $_FILES['audio']['tmp_name'][0];
    $file_name = basename($_FILES['audio']['name'][0]);
    //$file_name = preg_replace('/[^A-Za-z0-9\.\-]/', '', basename($_FILES['audio']['name']));
    $target_path = get_alarm_file_path($file_name);

    // log it
    send_to_audit_log(_("User uploaded audio '") . $file_name . "'", AUDITLOGTYPE_CHANGE);

    //echo "TEMP NAME: ".$_FILES['uploadedfile']['tmp_name']."<BR>\n";
    //echo "TARGET: ".$target_path."<BR>\n";

    $ret = array('status' => 'success');

    if (move_uploaded_file($tmp_loc, $target_path)) {

        // fix perms
        chmod($target_path, 0660);
        chgrp($target_path, "nagios");
    } else {

        // error
        $ret = array('error' => true, 'errmsg' => _('File was uploaded, but could not be moved to the correct location.'));
    }

    print json_encode($ret);

    return;
}

// Retrieve the NOC screen alarms from utils-sounds.inc.php
// return as JSON data.
function do_load_audio() {
    print json_encode(get_alarm_sounds());
}

// Requires a basename-able file + admin permissions
// Deletes the audio file from the server.
function do_delete_audio() {
    $fname = grab_request_var('file');
    if (empty($fname)) {
        print json_encode(array(
            'error' => true,
            'errmsg' => _('No file provided')
        ));
    }

    $fname = basename($fname);

    if (!is_admin()) {
        print json_encode(array(
            'error' => true,
            'errmsg' => _('Insufficient permission to delete audio clips')
        ));
        return;
    }

    delete_alarm_sound($fname);
    print json_encode(array('status' => 'success'));
}

function do_save_audio_prefs() {
    $events = grab_request_var('events', array());
    $sound = grab_request_var('sound', '');

    if (empty($events)) {
        print json_encode(array(
            'error' => true,
            'errmsg' => _('Please select at least one event from the list')
        ));
        return;
    }

    $prefs = get_user_meta(0, 'noc_sound_preferences', base64_encode(serialize(array())));
    $prefs = unserialize(base64_decode($prefs));

    for ($i = 0; $i < count($events); $i++) {
        $prefs[$events[$i]] = $sound;
    }

    $prefs = base64_encode(serialize($prefs));
    set_user_meta(0, 'noc_sound_preferences', $prefs);

    print json_encode(array(
        'status' => 'success'
    ));
}

function do_get_audio_prefs() {
    $prefs = get_user_meta(0, 'noc_sound_preferences', base64_encode(serialize(array())));
    $prefs = unserialize(base64_decode($prefs));
    print json_encode($prefs);
}